<?php
/*
Plugin Name: WP Finance
Plugin URI: http://mindomobile.com
Description: This plugin allows you to manage your financial data on Wordpress.
Version: 1.1.0
Author: Mindaugas Budreika
License: GPL2
*/

/**
* WP Finance Plugin Class
* 
* @package Wordpress
* @subpackage WP Finance Plugin
* @author Mindaugas Budreika
* @version 2
* @copyright WP Finance
*/

class WPFinancePlugin
{
   
    public function __construct()
    {
        add_action('admin_menu', array(&$this, 'admin_menu'));
        add_action('admin_init', array(&$this, 'admin_install'));
        add_action('admin_head', array(&$this, 'wpfinance_print_head'));
        
        add_shortcode('wpfinance', array(&$this, 'short_code_report'));
        
        // set up js   
        wp_enqueue_script('jquery');
        wp_enqueue_script('jquery-ui-core');
        wp_enqueue_script('jquery-ui-datepicker',
                          plugins_url('/js/ui.datepicker.min.js', __FILE__),
                          array('jquery','jquery-ui-core'));
        wp_enqueue_script('json', plugins_url('/js/json/json2.js', __FILE__));
        wp_enqueue_script('swfobject', plugins_url('/js/swfobject.js', __FILE__),
                          array('json'));
        
        wp_enqueue_style('datePicker', plugins_url('/css/datePicker.css', __FILE__));
        wp_enqueue_style('wpfinancereport', plugins_url('/css/report.css', __FILE__));
    }
    
    public function admin_install() {
        global $wpdb;
        
        /* Create database table */
        
        $wpdb->query(
            "CREATE TABLE IF NOT EXISTS `".$wpdb->prefix."finance` (".
               "`finance_id` int(11) NOT NULL AUTO_INCREMENT,".
               "`line` varchar(50) COLLATE utf8_unicode_ci NOT NULL,".
               "`description` text COLLATE utf8_unicode_ci NOT NULL,".
               "`date` date NOT NULL,".
               "`amount` double NOT NULL,".
               "`status` int(11) NOT NULL,".
               "PRIMARY KEY (`finance_id`)".
           ") ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci AUTO_INCREMENT=1;"
        );
    }
    
    public function admin_menu() {

        $this->is_admin = (bool)current_user_can('manage_options');    
        if (!$this->is_admin){ return; }
                
        /* add wp finance admin tab */
        add_menu_page('WP Finance', 'WP Finance', 'administrator', 8, 
            array(&$this, 'wpfinance_admin_wpfinance'));

        add_submenu_page(8, 'Report', 'Report', 'administrator', 1,
            array(&$this,'wpfinance_admin_report_options'));

        add_submenu_page(8, 'Chart', 'Chart', 'administrator', 2,
            array(&$this,'wpfinance_admin_chart'));
    }
    
    public function wpfinance_admin_report_options() {       
        if ($_GET['page'] != 1)
            return false;
        
        if ($_GET['action'] == "add-income") {
            $this->wpfinance_admin_add_income();
        } elseif ($_GET['action'] == "save-income") {
            $this->wpfinance_admin_save_income();
            $this->wpfinance_admin_report();
        }elseif ($_GET['action'] == "add-expenses") {
            $this->wpfinance_admin_add_expenses ();
        }elseif ($_GET['action'] == "save-expenses") {
            $this->wpfinance_admin_save_expenses();
            $this->wpfinance_admin_report();
        } elseif ($_GET['action'] == "remove-record") {
            $this->wpfinance_remove_record();
            $this->wpfinance_admin_report();
        } else {
            $this->wpfinance_admin_report();
        }
    }
      
    public function wpfinance_admin_wpfinance() {
        if ($_GET['page'] != 8)
            return false;
    
        /* save currency name */
        if ($_GET['action'] == "save-settings") {
            
            // Currency name
            if (preg_match("/^([a-z\ \$]+)$/sim", $_POST['currency'])) {
                if (get_option('wpfinance_currency') !=  $_POST['currency']) {
                    update_option('wpfinance_currency', $_POST['currency']);
                } else {
                    add_option('wpfinance_currency', $_POST['currency'], NULL, 'no');
                }                
            }
            
            // Currency position
            if (preg_match("/^([1-2]{1})$/", $_POST['currencyposition'])) {
                if (get_option('wpfinance_currencyposition') !=  $_POST['currencyposition']) {
                    update_option('wpfinance_currencyposition', $_POST['currencyposition']);
                } else {
                    add_option('wpfinance_currencyposition', $_POST['currencyposition'], NULL, 'no');
                } 
            }
        }
        
        $currency = get_option("wpfinance_currency");
        
        $currencyposition = get_option("wpfinance_currencyposition");
        if ($currencyposition != 1 && $currencyposition != 2) {
            $currencyposition = 2;
        }
        
        $submit_url = "http://".$_SERVER['HTTP_HOST'].$_SERVER['SCRIPT_NAME'];
        
        require_once(dirname(__FILE__) . "/templates/wp-finance.php");
    }
    
    public function wpfinance_admin_chart() {
        
        $submit_url = "http://".$_SERVER['HTTP_HOST'].$_SERVER['SCRIPT_NAME'];
        $c = plugins_url('/img/c.png', __FILE__);
        
        $date1 = (preg_match("/^([0-9\-]+)$/", $_GET['date1']))?$_GET['date1']:date("Y-m-d", strtotime("-1 month"));
        $date2 = (preg_match("/^([0-9\-]+)$/", $_GET['date2']))?$_GET['date2']:date("Y-m-d");
        
        require_once(dirname(__FILE__) . "/templates/chart.php");
    }
    
    public function wpfinance_admin_report() {
        global $wpdb;

        $date1 = (preg_match("/^([0-9\-]+)$/", $_GET['date1']))?$_GET['date1']:date("Y-m-d", strtotime("-1 month"));
        $date2 = (preg_match("/^([0-9\-]+)$/", $_GET['date2']))?$_GET['date2']:date("Y-m-d");

        $income = $wpdb->get_results("SELECT * FROM ".$wpdb->prefix."finance WHERE status='1' AND date >= '".$date1."' AND date <= '".$date2."' ORDER BY date ASC", ARRAY_A);
        $expenses = $wpdb->get_results("SELECT * FROM ".$wpdb->prefix."finance WHERE status='2' AND date LIKE '".$sql_month."%' ORDER BY date ASC", ARRAY_A);
        
        $submit_url = "http://".$_SERVER['HTTP_HOST'].$_SERVER['SCRIPT_NAME'];
        $c = plugins_url('/img/c.png', __FILE__);
        $currency = get_option("wpfinance_currency");
        
        $currencyposition = get_option("wpfinance_currencyposition");
        if ($currencyposition != 1 && $currencyposition != 2) {
            $currencyposition = 2;
        }
        
        require_once(dirname(__FILE__) . "/templates/report.php");
    }

    public function wpfinance_admin_add_income() {
        global $wpdb;
    
        $edit_id = $_GET['edit_id'];
        $url = "http://".$_SERVER['HTTP_HOST'].$_SERVER['SCRIPT_NAME'];
        
        $submit_url = $url."?page=1&amp;action=save-income";
        $c = plugins_url('/img/c.png', __FILE__);
    
        if (!empty($_GET['edit_id']) && preg_match("/^([0-9]+)$/", $_GET['edit_id'])) {
            $edit = $wpdb->get_row("SELECT * FROM ".$wpdb->prefix."finance WHERE finance_id='".$edit_id."' LIMIT 1", ARRAY_A);
        }
    
        require_once(dirname(__FILE__) . "/templates/add-income.php");
    }
    
    public function wpfinance_admin_add_expenses () {
        global $wpdb;
        
        $edit_id = $_GET['edit_id'];
        $submit_url = "http://".$_SERVER['HTTP_HOST'].$_SERVER['SCRIPT_NAME'];
        $c = plugins_url('/img/c.png', __FILE__);
                        
        if (!empty($_GET['edit_id']) && preg_match("/^([0-9]+)$/", $_GET['edit_id'])) {
            $edit = $wpdb->get_row("SELECT * FROM ".$wpdb->prefix."finance WHERE finance_id='".$edit_id."' LIMIT 1", ARRAY_A);
        }
    
        require_once(dirname(__FILE__) . "/templates/add-expenses.php");
    }
    
    public function wpfinance_admin_save_income() {
        global $wpdb;
        
        if (strlen($_POST['title']) == 0) return false;
              
        if (preg_match("/^([0-9]+)$/", $_POST['edit_id']) && !empty($_POST['edit_id'])) {
            // update database
            
            $wpdb->update( $wpdb->prefix.'finance',
                              array('line' => $_POST['title'],
                                    'description' => $_POST['description'],
                                    'date' => $_POST['date1'],
                                    'amount' => $_POST['amount'],
                                    'status' => 1),
                              array('finance_id' => $_POST['edit_id'] ),
                              array('%s', '%s', '%s', '%f', '%d'),
                              array('%d' ));
        } else {
            // insert new record to database
            
            $wpdb->insert( $wpdb->prefix.'finance',
                              array('finance_id' => $_POST['edit_id'],
                                    'line' => $_POST['title'],
                                    'description' => $_POST['description'],
                                    'date' => $_POST['date1'],
                                    'amount' => $_POST['amount'],
                                    'status' => 1),
                              array('%d', '%s', '%s', '%s', '%f', '%d'));
        }
    }
    
    public function wpfinance_admin_save_expenses() {
        global $wpdb;
              
        if (preg_match("/^([0-9]+)$/", $_POST['edit_id']) && !empty($_POST['edit_id'])) {
            // update database
            
            $wpdb->update( $wpdb->prefix.'finance',
                              array('line' => $_POST['title'],
                                    'description' => $_POST['description'],
                                    'date' => $_POST['date1'],
                                    'amount' => $_POST['amount'],
                                    'status' => 2),
                              array('finance_id' => $_POST['edit_id'] ),
                              array('%s', '%s', '%s', '%f', '%d'),
                              array('%d' ));
        } else {
            // insert new record to database
            
            $wpdb->insert( $wpdb->prefix.'finance',
                              array('finance_id' => $_POST['edit_id'],
                                    'line' => $_POST['title'],
                                    'description' => $_POST['description'],
                                    'date' => $_POST['date1'],
                                    'amount' => $_POST['amount'],
                                    'status' => 2),
                              array('%d', '%s', '%s', '%s', '%f', '%d'));
        }        
    }
    
    public function wpfinance_remove_record() {
        global $wpdb;
        
        if (preg_match("/^([0-9]+)$/", $_GET['edit_id'])) {
            $wpdb->query("DELETE FROM ".$wpdb->prefix."finance WHERE finance_id='".$_GET['edit_id']."' LIMIT 1");
        }
    }
    
    public function wpfinance_print_head() {
        global $wpdb;
        
        $swf_url = plugins_url('/open-flash-chart.swf', __FILE__);
        
        if ($_GET['page'] == 2) {
            require_once(dirname(__FILE__) . "/php-ofc-library/open-flash-chart.php");    
            
            $currencyposition = get_option("wpfinance_currencyposition");
            if ($currencyposition != 1 && $currencyposition != 2) {
                $currencyposition = 2;
            }
            
            $title = new title("Financial Report");

            $date1 = (preg_match("/^([0-9\-]+)$/", $_GET['date1']))?$_GET['date1']:date("Y-m-d", strtotime("-1 month"));
            $date2 = (preg_match("/^([0-9\-]+)$/", $_GET['date2']))?$_GET['date2']:date("Y-m-d");
            $edate1 = explode("-", $date1);
            $edate2 = explode("-", $date2);
        
            $data = array();
            $expenses_data = array();
            $diff = abs(strtotime($date2) - strtotime($date1));
            $days = ceil($diff/(60*60*24));
            
            $res = $wpdb->get_results("SELECT SUM(amount) AS amount, status, date FROM ".$wpdb->prefix.
                                      "finance WHERE date >= '".$date1."' AND date <= '".$date2."'".
                                      " GROUP BY date, status", ARRAY_A);

            for ($i=0; $i<count($res); $i++) {
                if ($res[$i]['status'] == 1) {
                    $db_i[$res[$i]['date']] = $res[$i]['amount'];
                } else {
                    $db_e[$res[$i]['date']] = $res[$i]['amount'];
                }
            }
            
            $db_i_max = (count($db_i) > 0)?max($db_i):1;  
            $db_e_max = (count($db_e) > 0)?max($db_e):1;
            
            for ($i=0; $i<=$days; $i++) {
                $item_date = date("Y-m-d", mktime(0, 0, 0, $edate2[1], $edate2[2], $edate2[0])-(60*60*24*$i));
                $eitem_date = explode("-", $item_date);
                
                if ($db_i[$item_date] > 0) {
                    $data[] = new scatter_value(mktime(0, 0, 0, $eitem_date[1], $eitem_date[2], $eitem_date[0]), ((double)$db_i[$item_date]));
                } else {
                    $data[] = new scatter_value(mktime(0, 0, 0, $eitem_date[1], $eitem_date[2], $eitem_date[0]), 0);
                }
                
                if ($db_e[$item_date] > 0) {
                    $expenses_data[] = new scatter_value(mktime(0, 0, 0, $eitem_date[1], $eitem_date[2], $eitem_date[0]), ((double)$db_e[$item_date]));
                } else {
                    $expenses_data[] = new scatter_value(mktime(0, 0, 0, $eitem_date[1], $eitem_date[2], $eitem_date[0]), 0);
                }
            }
            
            // dot            
            $d = new solid_dot();
            
            if ($currencyposition == 1) {
                $d->size(3)->halo_size(1)->colour('#0f7b12')->tooltip(get_option("wpfinance_currency").' #val#'.'<br>#date:Y-m-d#');
            }else{
                $d->size(3)->halo_size(1)->colour('#0f7b12')->tooltip('#val# '.get_option("wpfinance_currency").'<br>#date:Y-m-d#');
            }
            
            // line green
            $line = new line();
            $line->set_default_dot_style($d);
            $line->set_values( $data );
            $line->set_width( 2 );
            $line->set_colour( '#02a806' );
            $line->set_text("Income");

            /* expenses line */
            $d2 = new solid_dot();
            if ($currencyposition == 1) {
                $d2->size(3)->halo_size(1)->colour('#f21b1b')->tooltip(get_option("wpfinance_currency").' #val#'.'<br>#date:Y-m-d#');
            }else{
                $d2->size(3)->halo_size(1)->colour('#f21b1b')->tooltip('#val# '.get_option("wpfinance_currency").'<br>#date:Y-m-d#');
            }
            $line2 = new line();
            $line2->set_default_dot_style($d2);
            $line2->set_values($expenses_data);
            $line2->set_width(2);
            $line2->set_colour('#f21b1b');
            $line2->set_text("Expenses");

            /* end of expenses list */
         
            $chart = new open_flash_chart();
            $chart->set_title( $title );
            $chart->set_bg_colour('#FFFFFF');
            $chart->add_element( $line );
            $chart->add_element($line2);
            
            /* x axis */
            $x_title = new x_legend('Time');
            $x_title->set_style( '{font-size: 12px; color: #000000;}' );
            $chart->set_x_legend($x_title);
            
            $x = new x_axis();
            $x->set_range(
                mktime(0, 0, 0, $edate1[1], $edate1[2], $edate1[0]),
                mktime(0, 0, 0, $edate2[1], $edate2[2], $edate2[0])
            );
            $x_label = new x_axis_labels();
            $x_label->rotate(30);
            $x_label->visible_steps(2);
            $x_label->text('#date:Y-m-d#');
            $x_label->set_steps(86400);
            $x->set_labels($x_label);
            $x->set_steps(86400);
            $chart->set_x_axis($x);
            /* end of x axis*/
            
            $curr_string = (strlen(get_option("wpfinance_currency")) >0)?
                ", ".get_option("wpfinance_currency"):"";
                
            $y_title = new y_legend('Amount'.$curr_string);
            $y_title->set_style( '{font-size: 12px; color: #000000;}' );
            $chart->set_y_legend($y_title);
            
            $y = new y_axis();
            $y->set_range( 0, max($db_i_max, $db_e_max)+10, 5 );
            $chart->set_y_axis($y);
        }
        
        echo '<script type="text/javascript">';
        echo 'jQuery(function($){';
        echo '$( ".start-date" ).datepicker({dateFormat: \'yy-mm-dd\'});';
        echo '});';
  
        if ($_GET['page'] == 2) {
            echo 'swfobject.embedSWF(';
            echo '"'.$swf_url.'", "chart",';
            echo '"700", "400", "9.0.0"';
            echo ');';
        }
        
        echo '</script>';
        
        if ($_GET['page'] == 2) {
            echo '<script type="text/javascript">'.
                    'function ofc_ready()'.
                    '{}'.
                        
                    'function open_flash_chart_data()'.
                    '{'.
                        'return JSON.stringify(data);'.
                    '}'.
                    
                    'function findSWF(movieName) {'.
                        'if (navigator.appName.indexOf("Microsoft")!= -1) {'.
                            'return window[movieName];'.
                        '} else {'.
                            'return document[movieName];'.
                        '}'.
                    '}'.
                    
                    'var data = '.$chart->toPrettyString().';'.
                '</script>';
        }
    }
    
    public function short_code_report($atts, $content = null) {
        /*
         atts:
            from="[yyyy-mm-dd]" starting date for report
            to="[yyyy-mm-dd]" end date for report
            totals="true" show grand total table
        */

        global $wpdb;        
        
        if (preg_match("/^([0-9]{4}\-[0-9]{2}-[0-9]{2})?$/sim", $atts['from'])) {
            $start_date = " AND date >= '".$atts['from']."'";
        }
        
        if (preg_match("/^([0-9]{4}\-[0-9]{2}-[0-9]{2})?$/sim", $atts['to'])) {
            $end_date = " AND date <= '".$atts['to']."'";
        }
        
        $currency = get_option("wpfinance_currency");
        $currencyposition = get_option("wpfinance_currencyposition");
        if ($currencyposition != 1 && $currencyposition != 2) {
            $currencyposition = 2;
        }
        
        $records = $wpdb->get_results("SELECT * FROM ".$wpdb->prefix."finance WHERE 1".$start_date.$end_date, ARRAY_A);

        require_once(dirname(__FILE__) . "/templates/short-code.php");
        
        return $short_code_content;
    }
}

// Invoke class.
$rwp = new WPFinancePlugin();
?>
